@@ -1,9 +1,9 @@
 # coding=utf-8
 #
 # This file is part of Hypothesis, which may be found at
-# https://github.com/HypothesisWorks/hypothesis-python
+# https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2018 David R. MacIver
+# Most of this work is copyright (C) 2013-2019 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -11,50 +11,64 @@
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at http://mozilla.org/MPL/2.0/.
+# obtain one at https://mozilla.org/MPL/2.0/.
 #
 # END HEADER
 
 # pylint: skip-file
 
-from __future__ import division, print_function, absolute_import
+from __future__ import absolute_import, division, print_function
 
-import re
-import sys
-import math
-import time
 import array
 import codecs
-import platform
 import importlib
+import inspect
+import math
+import platform
+import re
+import sys
+import time
 from base64 import b64encode
 from collections import namedtuple
 
 try:
     from collections import OrderedDict, Counter
-except ImportError:  # pragma: no cover
-    from ordereddict import OrderedDict
-    from counter import Counter
+except ImportError:
+    from ordereddict import OrderedDict  # type: ignore
+    from counter import Counter  # type: ignore
+
+try:
+    from collections import abc
+except ImportError:
+    import collections as abc  # type: ignore
+
+if False:
+    from typing import Type, Tuple  # noqa
 
 
 PY2 = sys.version_info[0] == 2
 PY3 = sys.version_info[0] == 3
-PYPY = platform.python_implementation() == 'PyPy'
+PYPY = platform.python_implementation() == "PyPy"
 CAN_UNPACK_BYTE_ARRAY = sys.version_info[:3] >= (2, 7, 4)
+CAN_PACK_HALF_FLOAT = sys.version_info[:2] >= (3, 6)
 
-WINDOWS = platform.system() == 'Windows'
+WINDOWS = platform.system() == "Windows"
 
 if sys.version_info[:2] <= (2, 6):
-    raise ImportError(
-        'Hypothesis is not supported on Python versions before 2.7'
-    )
+    raise ImportError("Hypothesis is not supported on Python versions before 2.7")
 
 
 def bit_length(n):
     return n.bit_length()
 
 
+def quiet_raise(exc):
+    # Overridden by Py3 version, iff `raise XXX from None` is valid
+    raise exc
+
+
 if PY3:
+
     def str_to_bytes(s):
         return s.encode(a_good_encoding())
 
@@ -64,7 +78,7 @@ if PY3:
     text_type = str
     binary_type = bytes
     hrange = range
-    ARG_NAME_ATTRIBUTE = 'arg'
+    ARG_NAME_ATTRIBUTE = "arg"
     integer_types = (int,)
     hunichr = chr
 
@@ -75,21 +89,23 @@ if PY3:
         return s.isidentifier()
 
     def escape_unicode_characters(s):
-        return codecs.encode(s, 'unicode_escape').decode('ascii')
+        return codecs.encode(s, "unicode_escape").decode("ascii")
 
     def print_unicode(x):
         print(x)
 
-    exec("""
+    exec(
+        """
 def quiet_raise(exc):
     raise exc from None
-""")
+"""
+    )
 
     def int_from_bytes(data):
-        return int.from_bytes(data, 'big')
+        return int.from_bytes(data, "big")
 
     def int_to_bytes(i, size):
-        return i.to_bytes(size, 'big')
+        return i.to_bytes(size, "big")
 
     def to_bytes_sequence(ls):
         return bytes(ls)
@@ -104,6 +120,8 @@ def quiet_raise(exc):
 
     def benchmark_time():
         return time.monotonic()
+
+
 else:
     import struct
 
@@ -111,9 +129,12 @@ else:
         return hbytes(struct.pack(*args))
 
     if CAN_UNPACK_BYTE_ARRAY:
+
         def struct_unpack(fmt, string):
             return struct.unpack(fmt, string)
+
     else:
+
         def struct_unpack(fmt, string):
             return struct.unpack(fmt, str(string))
 
@@ -129,7 +150,7 @@ else:
         i = 0
         while i + 4 <= len(data):
             result <<= 32
-            result |= struct.unpack('>I', unpackable_data[i:i + 4])[0]
+            result |= struct.unpack(">I", unpackable_data[i : i + 4])[0]
             i += 4
         while i < len(data):
             result <<= 8
@@ -141,12 +162,13 @@ else:
         assert i >= 0
         result = bytearray(size)
         j = size - 1
+        arg = i
         while i and j >= 0:
             result[j] = i & 255
             i >>= 8
             j -= 1
         if i:
-            raise OverflowError('int too big to convert')
+            raise OverflowError("i=%r cannot be represented in %r bytes" % (arg, size))
         return hbytes(result)
 
     int_to_byte = chr
@@ -158,11 +180,9 @@ else:
         return s
 
     def int_to_text(i):
-        return str(i).decode('ascii')
+        return str(i).decode("ascii")
 
-    VALID_PYTHON_IDENTIFIER = re.compile(
-        r"^[a-zA-Z_][a-zA-Z0-9_]*$"
-    )
+    VALID_PYTHON_IDENTIFIER = re.compile(r"^[a-zA-Z_][a-zA-Z0-9_]*$")
 
     def isidentifier(s):
         return VALID_PYTHON_IDENTIFIER.match(s)
@@ -186,7 +206,7 @@ else:
                 return xrange(start_or_finish, finish, step)
         except OverflowError:
             if step == 0:
-                raise ValueError(u'step argument may not be zero')
+                raise ValueError(u"step argument may not be zero")
             if step is None:
                 step = 1
             if finish is not None:
@@ -196,34 +216,35 @@ else:
                 finish = start_or_finish
             assert step != 0
             if step > 0:
+
                 def shimrange():
                     i = start
                     while i < finish:
                         yield i
                         i += step
+
             else:
+
                 def shimrange():
                     i = start
                     while i > finish:
                         yield i
                         i += step
+
             return shimrange()
 
-    ARG_NAME_ATTRIBUTE = 'id'
+    ARG_NAME_ATTRIBUTE = "id"
     integer_types = (int, long)
     hunichr = unichr
 
     def escape_unicode_characters(s):
-        return codecs.encode(s, 'string_escape')
+        return codecs.encode(s, "string_escape")
 
     def print_unicode(x):
         if isinstance(x, unicode):
             x = x.encode(a_good_encoding())
         print(x)
 
-    def quiet_raise(exc):
-        raise exc
-
     def benchmark_time():
         return time.time()
 
@@ -235,18 +256,22 @@ else:
 # into a string file path using the appropriate encoding. See
 # https://bitbucket.org/ned/coveragepy/issues/602/ for more information.
 if PY2:
+
     def encoded_filepath(filepath):
         if isinstance(filepath, text_type):
             return filepath.encode(sys.getfilesystemencoding())
         else:
             return filepath
+
+
 else:
+
     def encoded_filepath(filepath):
         return filepath
 
 
 def a_good_encoding():
-    return 'utf-8'
+    return "utf-8"
 
 
 def to_unicode(x):
@@ -262,81 +287,112 @@ def qualname(f):
     except AttributeError:
         pass
     try:
-        return f.im_class.__name__ + '.' + f.__name__
+        return f.im_class.__name__ + "." + f.__name__
     except AttributeError:
         return f.__name__
 
 
+try:
+    import typing
+except ImportError:
+    typing_root_type = ()  # type: Tuple[type, ...]
+    ForwardRef = None
+else:
+    try:
+        # These types are new in Python 3.7, but also (partially) backported to the
+        # typing backport on PyPI.  Use if possible; or fall back to older names.
+
+        typing_root_type = (typing._Final, typing._GenericAlias)  # type: ignore
+        ForwardRef = typing.ForwardRef  # type: ignore
+    except AttributeError:
+        typing_root_type = (typing.TypingMeta, typing.TypeVar)  # type: ignore
+        ForwardRef = typing._ForwardRef  # type: ignore
+
+
 if PY2:
-    FullArgSpec = namedtuple('FullArgSpec', 'args, varargs, varkw, defaults, '
-                             'kwonlyargs, kwonlydefaults, annotations')
+    FullArgSpec = namedtuple(
+        "FullArgSpec",
+        "args, varargs, varkw, defaults, kwonlyargs, kwonlydefaults, annotations",
+    )
 
     def getfullargspec(func):
-        import inspect
         args, varargs, varkw, defaults = inspect.getargspec(func)
-        return FullArgSpec(args, varargs, varkw, defaults, [], None,
-                           getattr(func, '__annotations__', {}))
+        return FullArgSpec(
+            args,
+            varargs,
+            varkw,
+            defaults,
+            [],
+            None,
+            getattr(func, "__annotations__", {}),
+        )
+
+
 else:
     from inspect import getfullargspec, FullArgSpec
 
 
 if sys.version_info[:2] < (3, 6):
+
     def get_type_hints(thing):
         try:
             spec = getfullargspec(thing)
             return {
-                k: v for k, v in spec.annotations.items()
+                k: v
+                for k, v in spec.annotations.items()
                 if k in (spec.args + spec.kwonlyargs) and isinstance(v, type)
             }
         except TypeError:
             return {}
+
+
 else:
+    import typing
+
     def get_type_hints(thing):
         try:
-            import typing
             return typing.get_type_hints(thing)
         except TypeError:
             return {}
 
 
-importlib_invalidate_caches = getattr(
-    importlib, 'invalidate_caches', lambda: ())
+importlib_invalidate_caches = getattr(importlib, "invalidate_caches", lambda: ())
 
 
 if PY2:
     CODE_FIELD_ORDER = [
-        'co_argcount',
-        'co_nlocals',
-        'co_stacksize',
-        'co_flags',
-        'co_code',
-        'co_consts',
-        'co_names',
-        'co_varnames',
-        'co_filename',
-        'co_name',
-        'co_firstlineno',
-        'co_lnotab',
-        'co_freevars',
-        'co_cellvars',
+        "co_argcount",
+        "co_nlocals",
+        "co_stacksize",
+        "co_flags",
+        "co_code",
+        "co_consts",
+        "co_names",
+        "co_varnames",
+        "co_filename",
+        "co_name",
+        "co_firstlineno",
+        "co_lnotab",
+        "co_freevars",
+        "co_cellvars",
     ]
 else:
     CODE_FIELD_ORDER = [
-        'co_argcount',
-        'co_kwonlyargcount',
-        'co_nlocals',
-        'co_stacksize',
-        'co_flags',
-        'co_code',
-        'co_consts',
-        'co_names',
-        'co_varnames',
-        'co_filename',
-        'co_name',
-        'co_firstlineno',
-        'co_lnotab',
-        'co_freevars',
-        'co_cellvars',
+        "co_argcount",
+        "co_kwonlyargcount",
+        "co_nlocals",
+        "co_stacksize",
+        "co_flags",
+        "co_code",
+        "co_consts",
+        "co_names",
+        "co_varnames",
+        "co_filename",
+        "co_name",
+        "co_firstlineno",
+        "co_lnotab",
+        "co_freevars",
+        "co_cellvars",
     ]
 
 
@@ -345,21 +401,19 @@ def update_code_location(code, newfile, newlineno):
 
     Why do we want to do this? It's for really shallow reasons involving
     hiding the hypothesis_temporary_module code from test runners like
-    py.test's verbose mode. This is a vastly disproportionate terrible
+    pytest's verbose mode. This is a vastly disproportionate terrible
     hack that I've done purely for vanity, and if you're reading this
     code you're probably here because it's broken something and now
     you're angry at me. Sorry.
     """
-    unpacked = [
-        getattr(code, name) for name in CODE_FIELD_ORDER
-    ]
-    unpacked[CODE_FIELD_ORDER.index('co_filename')] = newfile
-    unpacked[CODE_FIELD_ORDER.index('co_firstlineno')] = newlineno
+    unpacked = [getattr(code, name) for name in CODE_FIELD_ORDER]
+    unpacked[CODE_FIELD_ORDER.index("co_filename")] = newfile
+    unpacked[CODE_FIELD_ORDER.index("co_firstlineno")] = newlineno
     return type(code)(*unpacked)
 
 
 class compatbytes(bytearray):
-    __name__ = 'bytes'
+    __name__ = "bytes"
 
     def __init__(self, *args, **kwargs):
         bytearray.__init__(self, *args, **kwargs)
@@ -369,7 +423,7 @@ class compatbytes(bytearray):
         return bytearray.__str__(self)
 
     def __repr__(self):
-        return 'compatbytes(b%r)' % (str(self),)
+        return "compatbytes(b%r)" % (str(self),)
 
     def __hash__(self):
         if self.__hash is None:
@@ -387,7 +441,7 @@ class compatbytes(bytearray):
         for i, v in enumerate(self):
             if v == value:
                 return i
-        raise ValueError('Value %r not in sequence %r' % (value, self))
+        raise ValueError("Value %r not in sequence %r" % (value, self))
 
     def __add__(self, value):
         assert isinstance(value, compatbytes)
@@ -410,7 +464,7 @@ class compatbytes(bytearray):
         else:
             return r
 
-    __setitem__ = None
+    __setitem__ = None  # type: ignore
 
     def join(self, parts):
         result = bytearray()
@@ -436,38 +490,42 @@ else:
     string_types = (str,)
 
 
-EMPTY_BYTES = hbytes(b'')
+EMPTY_BYTES = hbytes(b"")
 
 if PY2:
+
     def to_str(s):
         if isinstance(s, unicode):
             return s.encode(a_good_encoding())
         assert isinstance(s, str)
         return s
+
+
 else:
+
     def to_str(s):
         return s
 
 
 def cast_unicode(s, encoding=None):
     if isinstance(s, bytes):
-        return s.decode(encoding or a_good_encoding(), 'replace')
+        return s.decode(encoding or a_good_encoding(), "replace")
     return s
 
 
 def get_stream_enc(stream, default=None):
-    return getattr(stream, 'encoding', None) or default
+    return getattr(stream, "encoding", None) or default
 
 
 def implements_iterator(it):
     """Turn things with a __next__ attribute into iterators on Python 2."""
-    if PY2 and not hasattr(it, 'next') and hasattr(it, '__next__'):
+    if PY2 and not hasattr(it, "next") and hasattr(it, "__next__"):
         it.next = it.__next__
     return it
 
 
 if PY3:
-    FileNotFoundError = FileNotFoundError
+    FileNotFoundError = FileNotFoundError  # type: Type[IOError]
 else:
     FileNotFoundError = IOError
 
@@ -475,7 +533,7 @@ else:
 # an existing file where you're not allowed to. This is rather less consistent
 # between versions than might be hoped.
 if PY3:
-    FileExistsError = FileExistsError
+    FileExistsError = FileExistsError  # type: Type[IOError]
 
 elif WINDOWS:
     FileExistsError = WindowsError
@@ -487,22 +545,24 @@ else:
     FileExistsError = None
 
 
-if PY2:
-    # Under Python 2, math.floor and math.ceil return floats, which cannot
-    # represent large integers - eg `float(2**53) == float(2**53 + 1)`.
-    # We therefore implement them entirely in (long) integer operations.
-    def floor(x):
-        if int(x) != x and x < 0:
-            return int(x) - 1
-        return int(x)
+# Under Python 2, math.floor and math.ceil return floats, which cannot
+# represent large integers - eg `float(2**53) == float(2**53 + 1)`.
+# We therefore implement them entirely in (long) integer operations.
+# We use the same trick on Python 3, because Numpy values and other
+# custom __floor__ or __ceil__ methods may convert via floats.
+# See issue #1667, Numpy issue 9068.
+def floor(x):
+    y = int(x)
+    if y != x and x < 0:
+        return y - 1
+    return y
 
-    def ceil(x):
-        if int(x) != x and x > 0:
-            return int(x) + 1
-        return int(x)
-else:
-    floor = math.floor
-    ceil = math.ceil
+
+def ceil(x):
+    y = int(x)
+    if y != x and x > 0:
+        return y + 1
+    return y
 
 
 try:
@@ -512,25 +572,44 @@ except ImportError:
 
 
 if PY2:
+
     def b64decode(s):
         from base64 import b64decode as base
+
         return hbytes(base(s))
+
+
 else:
     from base64 import b64decode
 
 
-_cases = []
+try:
+    from django.test import TransactionTestCase
+
+    def bad_django_TestCase(runner):
+        if runner is None:
+            return False
+        if not isinstance(runner, TransactionTestCase):
+            return False
+
+        from hypothesis.extra.django._impl import HypothesisTestCase
+
+        return not isinstance(runner, HypothesisTestCase)
 
 
-def bad_django_TestCase(runner):
-    if runner is None:
+except Exception:
+    # Can't use ImportError, because of e.g. Django config errors
+    def bad_django_TestCase(runner):
         return False
-    if not _cases:
-        try:
-            from django.test import TransactionTestCase
-            from hypothesis.extra.django import HypothesisTestCase
-            _cases[:] = TransactionTestCase, HypothesisTestCase
-        except Exception:
-            # Can't use ImportError, because of e.g. Django config errors
-            _cases[:] = (), type
-    return isinstance(runner, _cases[0]) and not isinstance(runner, _cases[1])
+
+
+if PY2:
+    LIST_CODES = ("q", "Q", "O")
+else:
+    LIST_CODES = ("O",)
+
+
+def array_or_list(code, contents):
+    if code in LIST_CODES:
+        return list(contents)
+    return array.array(code, contents)
